#!/usr/bin/perl

#
# Author: Ahfyth
#

use strict;
use warnings;

if( $#ARGV < 1 )
{
	print STDERR "\nUsage: $0 <in.data> <output.prefix>\n";
	print STDERR "\nThis program is designed to split the reads into SHORT and LONG categories.\n\n";
	exit 2;
}

## parameters for SHORT and LONG categories
my $C1_s = 90;
my $C1_e = 150;
my $C2_s = 170;
#my $C2_e = 250;
my $C2_e = 200;

my ($C1, $C2) = ('', '');

if( $ARGV[0] =~ /.bed$/ )
{
	open IN, "$ARGV[0]" or die( "$!" );
	while( <IN> )
	{
		chomp;
		my @l = split /\t/;	##chr start end
		next unless $l[0]=~/\d+$/;
		my $len = $l[2]-$l[1];
		if( $len<=$C1_e && $len>=$C1_s )    { $C1 .= "$l[0]\t$l[1]\t$l[2]\t$len\n"; }
		elsif( $len<=$C2_e && $len>=$C2_s ) { $C2 .= "$l[0]\t$l[1]\t$l[2]\t$len\n"; }
	}
	close IN;

	open OUT, ">$ARGV[1].short.dat" or die( "$!" );
	print OUT $C1;
	close OUT;
	open OUT, ">$ARGV[1].long.dat"  or die( "$!" );
	print OUT $C2;
	close OUT;

	exit 0;
}

my ($chrcol, $poscol, $seqcol);	##chr-column, position-column and sequence-column, these two depends on the file format
if( $ARGV[0] =~ /.soap$/ || $ARGV[0] =~ /.bsalign$/ )
{
	$chrcol = 7;
	$poscol = 8;
	$seqcol = 1;
}
elsif( $ARGV[0] =~ /.sam$/ )
{
	$chrcol = 2;
	$poscol = 3;
	$seqcol = 9;
}
else
{
	print STDERR "ERROR: Only bed/soap/bsalign/sam format file is acceptable.\n";
	exit 100;
}


open IN, "$ARGV[0]" or die( "$!" );
my ( $r1, $r2 );
my ( $s, $e, $len );
while( $r1 = <IN> )
{
	$r2 = <IN>;
	my @l1 = split /\t/, $r1;
	next unless $l1[ $chrcol ] =~ /\d+$/;	## only autosomes are considered
	my @l2 = split /\t/, $r2;
	if( $l1[ $poscol ] <= $l2[ $poscol ] )
	{
		$s = $l1[ $poscol ];
		$e = $l2[ $poscol ] + length( $l2[ $seqcol ] );
	}
	else
	{
		$s = $l2[ $poscol ];
		$e = $l1[ $poscol ] + length( $l1[ $seqcol ] );
	}
	$len = $e - $s;

	if( $len<=$C1_e && $len>=$C1_s )	{ $C1 .= "$l1[ $chrcol ]\t$s\t$e\n"; }
	elsif( $len<=$C2_e && $len>=$C2_s )	{ $C2 .= "$l1[ $chrcol ]\t$s\t$e\n"; }
}
close IN;

open OUT, ">$ARGV[1].short.dat" or die( "$!" );
print OUT $C1;
close OUT;
open OUT, ">$ARGV[1].long.dat"  or die( "$!" );
print OUT $C2;
close OUT;

